import os, json
import geoh

__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))
geojson = json.loads(open(os.path.join(__location__, './tests/geojson-sf.json')).read())

geohashes = geoh.geohashes(geojson=geojson, precision=5)
print geohashes



import os
import json
import pytest
import geoh

@pytest.fixture
def geojson_sf():
  __location__ = os.path.realpath(os.path.join(
      os.getcwd(), os.path.dirname(__file__)))
  geojson = json.loads(
    open(os.path.join(__location__, './geojson-sf.json')).read())
  return geojson

@pytest.fixture
def geojson_none():
  return {
    'type': 'FeatureCollection', 
    'features': [None]
  }

@pytest.fixture
def malformed_geojson():
    return {
        'type': 'FeatureCollection',
        'features': [{
          'geometry': None,
          'type': 'Polygon',
        }]
    }

@pytest.mark.parametrize("precision", [1, 2 ,3, 4, 5, 6])
def test_sf(geojson_sf, precision):
  geohashes = geoh.geohashes(geojson=geojson_sf, precision=precision)
  assert(len(geohashes) > 0)

@pytest.mark.parametrize("precision", [1, 2, 3, 4, 5, 6])
def test_with_none_geojson(geojson_none, precision):
  geohashes = geoh.geohashes(geojson=geojson_none, precision=precision)
  assert(geohashes == [])


@pytest.mark.parametrize("precision", [1, 2, 3, 4, 5, 6])
def test_with_malformed_geojson(malformed_geojson, precision):
  geohashes = geoh.geohashes(geojson=malformed_geojson, precision=precision)
  assert(geohashes == [])



from distutils.core import setup
from setuptools import find_packages

required = [
    "pandas>=0.15.2",
    "geopandas>=0.2",
    "shapely>=1.5.13",
    "python-geohash>=0.8.5"
]

setup(
    name="geoh",
    version="0.3",
    author="Mathieu Ripert",
    author_email="mathieu@instacart.com",
    url="https://github.com/mathieuripert/geoh",
    license="MIT",
    packages=find_packages(),
    package_dir={"geoh": "geoh"},
    description="Transform a geoJSON into a list of geohashes that intersect with it",
    install_requires=required,
    classifiers=[],

)



from shapely.geometry import shape, MultiPolygon, mapping
import pandas as pd
import geopandas as gpd
import geohash as gh

BASE32 = "0123456789bcdefghjkmnpqrstuvwxyz"

def geohashes(geojson={}, precision=6, start_precision=2):
  """
    Return the list of geohashes that interects the geojson.
  """
  polygon = _polygon_from_geojson(geojson)
  if not polygon:
    return []

  p = min(start_precision, precision)
  center_geohash = get_center_geohash(polygon, precision=p)
  _geohashes = [center_geohash] + gh.neighbors(center_geohash)
  _geohashes = geohashes_polygon_intersection(polygon, _geohashes)

  while p < precision:
    _geohashes = _generate_inner_geohashes_for_geohashes(_geohashes)
    _geohashes = geohashes_polygon_intersection(polygon, _geohashes)
    p += 1

  return _geohashes

def get_center_geohash(polygon, precision=2):
  centroid = mapping(polygon.centroid)["coordinates"]
  return gh.encode(centroid[1], centroid[0], precision=precision)

def geohashes_polygon_intersection(polygon, geohashes=[]):
  df = gpd.GeoDataFrame(geohashes, columns=["geohash"])
  df["geometry"] = df.apply(lambda x: _geohash_to_shape(x["geohash"]), axis=1)
  return list(df[df.intersects(polygon)].geohash)

def _generate_inner_geohashes_for_geohashes(geohashes=[]):
  return _flatten(map(_generate_inner_geohashes_for_geohash, geohashes))

def _generate_inner_geohashes_for_geohash(geohash=""):
  return ["%s%s" % (geohash, l) for l in list(BASE32)]

def _flatten(lyst):
  return [item for sublist in lyst for item in sublist]

def _multi_polygon_to_polygons(geom):
  if geom.geom_type == 'MultiPolygon':
    return list(geom)
  return [geom]

def _polygon_from_geojson(geojson={}):
  if geojson.get("type", None) == "FeatureCollection":
    features = [f for f in geojson.get("features", []) if f is not None]
    df = pd.DataFrame(features, columns=["geometry"])
    df["polygon"] = df[pd.notnull(df.geometry)].geometry.map(lambda x: shape(x))
    df = df[pd.notnull(df.polygon)]
    if len(df.index):
      return MultiPolygon(_flatten(map(_multi_polygon_to_polygons, df["polygon"].values)))
    else:
      return None
  elif geojson.get("type", None) == "Feature":
    return shape(geojson.get("geometry", None))
  return None

def _geohash_to_shape(geohash):
    box = gh.bbox(geohash)
    coords = [
        [box["w"], box["s"]],
        [box["w"], box["n"]],
        [box["e"], box["n"]],
        [box["e"], box["s"]],
        [box["w"], box["s"]],
    ]
    return shape({"type": "Polygon", "coordinates": [coords]})
